;Chong Carl
;SPARC CHIRP.ASM

.nolist
.include "c:\users\cc88\8515def.inc"
.list

;define registers
.def	save		=r1	;Save location for the status resister
.def	reload		=r2	;Value to set timer 0 to 1 Msec.

.def	temp		=r16	;Temporary register
.def 	time1		=r18	;Elapsed time for task 1
.def	cnt_sylnum	=r20	;Count variable for number of syllables
.def	MakeNoise	=r24	;Make Noise Flag
.def	State		=r25	;State definition
.def	cnt_sdur	=r26	;Count variable for syllable duration
.def	cnt_qdur	=r27	;Count variable for quiet duration
.def	cnt_silence	=r28	;Count variable	for silence between chirps

;Specify the interval for the tasks to execute:
.equ	t1		=30	;30 mSec 
.equ	noise		=0	;Make noise
.equ	quiet		=1	;Quiet in between syllable
.equ	wait		=2	;Quiet in between chirp
.equ	syllable	=4	;Number of syllables
.equ	s_duration	=15	;Syllable duration
.equ	r_interval	=15	;Repeat interval
.
;*************************************
;Call #	Freq	Period (.25us)	Enter ISR Period
;4	3000	1333		667
;*************************************

; Initialization
.cseg
.org $0000
	rjmp 	RESET		;Reset entry vector
	reti		
	reti
	reti
	rjmp 	T1CompA		;Timer1 compare
	reti
	reti
	rjmp 	TIMER0		;Timer0
	reti
	reti		
	reti
	reti	
	reti


RESET:
	ldi	Temp, LOW(RAMEND)	;Setup stack pointer
	out 	SPL, Temp
	ldi	Temp, HIGH(RAMEND)
	out	SPH, Temp

	;set up the PORTs
	ser	Temp		;Set PORTC to be
	out	DDRC,Temp	; all outputs
	ldi	Temp, 0xff	; and turn on all bits
	out	PORTC, Temp

	;set up timer 0 for 1 mSec ticks
	ldi	temp, 3		;Prescalar to 64
	out 	TCCR0, Temp
	ldi	temp,256-127	;Preload timer since
	mov	reload, temp
	out 	TCNT0, reload	;62.5 x (64x.25) microSec = 1.0 mSec.

	;setup timer 1 for reset on compare match at full clock rate
	ldi	temp, 0b00001001
	out	TCCR1B, temp
	ldi	temp, 0		;Now set timer 1 to some very high rate
	out	OCR1AH, temp	; which will be above audio
	ldi	temp, 40	; and therefore quiet
	out	OCR1AL, temp

	;enable both timer interrupts
	ldi	Temp,0b01000010	
	out	TIMSK, Temp

	;initialize task timers
	ldi	time1, t1	;mSec

	;initialize to chirp 1
	ldi	State,0		;Iinititalize state to 0

	;Start the clock ticking
	sei			;Enable all interrupts	

;*************************************
;Now start scheduling events.
;This is the main program loop.
;All tasks are subroutines called from here when
;their respective timers reach zero.

Sched:		
tsk1:	tst	time1		;Test for first task ready
	brne	tskend		;If not then skip it
	rcall	Task1

tskend:	rjmp	Sched
	
;*************************************
;This is task 1
;It checks for which button was pushed and enables task 2 with
; the appropriate variable initializations
;This logic also checks to see whether or not more than one button
; was pressed. If so, do not do anything.

task1:	ldi	time1, t1	;Reload task timer
	ldi	r30,0x8A	;Chirp + silence duration
	ldi	r31,0x2
	ldi	temp, 0x9B	;Set T1CompA to
	out	OCR1AL, temp
	ldi	temp, 0x2
	out	OCR1AH, temp
	ret
	
;*************************************
;timer 0 ISR (timer-zero overflow) 
;Enters every 1.0 mSec

TIMER0:	in	save, SREG
	out 	TCNT0, Reload	;Keeps clock ticking at 1 mSec
	
	dec	time1
	cpi	State,noise
	breq	_MakeNoise
	cpi	State,quiet
	breq	_Quiet
	cpi	State,wait
	breq	_Wait
	rjmp	t0end

_MakeNoise:			;Make noise if flag is not set
	tst	cnt_sdur
	breq	_MakeNoiseDone
	dec	cnt_sdur
	rjmp	t0end

_MakeNoiseDone:			;Stop making noise
	ldi	cnt_sdur,s_duration
	ldi	State,quiet
	ldi	MakeNoise,0
	rjmp	t0end
		
_Quiet:	tst	cnt_qdur	;Kill the cricket noise
	breq	_QuietDone1
	dec	cnt_qdur
	rjmp	t0end

_QuietDone1:
	ldi	cnt_qdur,r_interval
	tst	cnt_sylnum
	breq	_QuietDone2
	dec	cnt_sylnum
	ldi	State,noise
	ldi	MakeNoise,0xff
	rjmp	t0end
		
_QuietDone2:
	ldi	cnt_sylnum,syllable
	ldi	State,wait
	rjmp	t0end

_Wait:	tst	r28
	breq	_Wait2
	sbiw	cnt_silence,1
	rjmp	t0end

_Wait2:	tst	r29
	breq	_WaitDone
	sbiw	cnt_silence,1
	rjmp	t0end

_WaitDone:
	mov	r28,r30
	mov	r29,r31	
	ldi	State,noise
	ldi	MakeNoise,0xff	
	rjmp	t0end

t0end:	out	SREG, save
	reti			;Back to backgound 

;*************************************
;timer 1 compare match ISR 
;Enters every 1/2 cycle of the audio and toggles the output bit

T1CompA:in	save, SREG	;Save cpu state
	tst	MakeNoise	;Should we make noise?
	breq	T1Done
	push	temp
	in	temp, PORTC	;Get the output
	com	temp		;Invert it
	out	PORTC, temp	;Back out again
	pop	temp
	out	SREG, save	;Restore cpu state

T1Done:	reti
;*************************************